import {
  validateQualifier,
} from '@loafer/pouches/util/qualifier';

import PouchDefinition from '@loafer/pouches/factory/config/PouchDefinition';
import PouchDefinitionRegistry from '@loafer/pouches/factory/registry/PouchDefinitionRegistry';
import DefaultSingletonPouchRegistry from '@loafer/pouches/factory/implement/DefaultSingletonPouchRegistry';



abstract class AbstractPouchFactory extends DefaultSingletonPouchRegistry implements PouchDefinitionRegistry {
  protected pouchDefinitionMap: Map<string | symbol, Map<Object, PouchDefinition>>;
  public constructor() {
    super();
    this.pouchDefinitionMap = new Map();
  }

  public registerPouchDefinition(pouchDefinition: PouchDefinition): void {
    let type = pouchDefinition.Type;
    let qualifier = validateQualifier(type, pouchDefinition.Qualifier);

    if (this.hasPouchDefinition(type, qualifier)) {
      throw new Error(`Pouch Definition[${type.constructor.name}:${qualifier}] is exist already`);
    }
    let map = this.pouchDefinitionMap.get(qualifier);
    if (undefined === map) {
      map = new Map();
      this.pouchDefinitionMap.set(qualifier, map);
    }
    map.set(type, pouchDefinition);
  }
  public removePouchDefinition(type: Object, qualifier?: string | symbol): void {
    const _qualifier = validateQualifier(type, qualifier);
    if (!this.hasPouchDefinition(type, _qualifier)) {
      throw new Error(`Pouch Definition[${type.constructor.name}:${qualifier}] is not exist`);
    }
    this.pouchDefinitionMap.get(_qualifier).delete(type);
    if (0 === this.pouchDefinitionMap.get(_qualifier).size) {
      this.pouchDefinitionMap.delete(_qualifier);
    }
  }
  public getPouchDefinition(type: Object, qualifier?: string | symbol): PouchDefinition {
    const _qualifier = validateQualifier(type, qualifier);

    if (!this.pouchDefinitionMap.has(_qualifier)) {
      return undefined;
    }
    return this.pouchDefinitionMap.get(_qualifier).get(type);
  }
  public hasPouchDefinition(type: Object, qualifier?: string | symbol): boolean {
    return undefined === this.getPouchDefinition(type, qualifier) ? false : true;
  }



}

export default AbstractPouchFactory;
