import {
  Construtorable,
} from '@loafer/core/constants';

import SingletonPouchRegistry from '@loafer/pouches/factory/config/SingletonPouchRegistry';

import {
  validateQualifier,
} from '@loafer/pouches/util/qualifier';

class DefaultSingletonPouchRegistry implements SingletonPouchRegistry {
  protected singletonInstanceMap: Map<string | symbol, Map<Object, any>>;

  protected constructor() {
  }

  public registerSingleton(pouch: any, qualifier?: string | symbol): void {
    let type: Object = Object.getPrototypeOf(pouch);
    const _qualifier = validateQualifier(type, qualifier);

    if (this._hasSingleton(type, _qualifier)) {
      throw new Error(`Pouch Definition[${type.constructor.name}:${qualifier}] is exist already`);
    }
    let map = this.singletonInstanceMap.get(_qualifier);
    if (undefined === map) {
      map = new Map();
      this.singletonInstanceMap.set(_qualifier, map);
    }
    map.set(type, pouch);
  }
  public getSingleton<T>(target: Construtorable<T>, qualifier?: string | symbol): T {
    let type = target.prototype;
    return this._getSingleton(type, qualifier);
  }
  public hasSingleton<T>(target: Construtorable<T>, qualifier?: string | symbol): boolean {
    return this._hasSingleton(target.prototype, qualifier);
  }

  private _getSingleton<T>(type: Object, qualifier?: string | symbol): T {
    const _qualifier = validateQualifier(type, qualifier);

    if (!this.singletonInstanceMap.has(_qualifier)) {
      return undefined;
    }
    return this.singletonInstanceMap.get(_qualifier).get(type);
  }

  private _hasSingleton(type: Object, qualifier?: string | symbol): boolean {
    return undefined === this._getSingleton(type, qualifier) ? false : true;
  }
}

export default DefaultSingletonPouchRegistry;
